package javax.swing.plaf.metal;

import java.awt.Container;
import java.awt.LayoutManager;
import java.awt.event.ActionEvent;
import javax.swing.AbstractAction;
import javax.swing.filechooser.FileFilter;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Arrays;
import sun.awt.shell.ShellFolder;
import java.util.Vector;
import javax.swing.ComboBoxModel;
import javax.swing.AbstractListModel;
import java.awt.Graphics;
import javax.swing.Icon;
import java.awt.Component;
import javax.swing.DefaultListCellRenderer;
import java.awt.event.MouseAdapter;
import javax.swing.plaf.basic.BasicDirectoryModel;
import java.awt.event.MouseListener;
import javax.swing.JList;
import javax.swing.event.ListSelectionEvent;
import java.awt.ComponentOrientation;
import java.io.File;
import javax.swing.event.ListSelectionListener;
import javax.swing.plaf.ActionMapUIResource;
import javax.swing.ActionMap;
import javax.swing.SwingUtilities;
import sun.swing.SwingUtilities2;
import java.util.Locale;
import javax.swing.filechooser.FileSystemView;
import java.awt.event.FocusEvent;
import java.awt.event.FocusAdapter;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import javax.swing.ButtonGroup;
import javax.swing.UIManager;
import javax.swing.Box;
import javax.swing.BoxLayout;
import java.awt.BorderLayout;
import javax.swing.border.EmptyBorder;
import javax.swing.JFileChooser;
import javax.swing.plaf.ComponentUI;
import javax.swing.JComponent;
import java.awt.Insets;
import java.awt.Dimension;
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.JToggleButton;
import sun.swing.FilePane;
import javax.swing.JTextField;
import javax.swing.Action;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.plaf.basic.BasicFileChooserUI;

public class MetalFileChooserUI extends BasicFileChooserUI
{
    private JLabel lookInLabel;
    private JComboBox directoryComboBox;
    private MetalFileChooserUI.DirectoryComboBoxModel directoryComboBoxModel;
    private Action directoryComboBoxAction;
    private MetalFileChooserUI.FilterComboBoxModel filterComboBoxModel;
    private JTextField fileNameTextField;
    private FilePane filePane;
    private JToggleButton listViewButton;
    private JToggleButton detailsViewButton;
    private JButton approveButton;
    private JButton cancelButton;
    private JPanel buttonPanel;
    private JPanel bottomPanel;
    private JComboBox filterComboBox;
    private static final Dimension hstrut5;
    private static final Dimension hstrut11;
    private static final Dimension vstrut5;
    private static final Insets shrinkwrap;
    private static int PREF_WIDTH;
    private static int PREF_HEIGHT;
    private static Dimension PREF_SIZE;
    private static int MIN_WIDTH;
    private static int MIN_HEIGHT;
    private static int LIST_PREF_WIDTH;
    private static int LIST_PREF_HEIGHT;
    private static Dimension LIST_PREF_SIZE;
    private int lookInLabelMnemonic;
    private String lookInLabelText;
    private String saveInLabelText;
    private int fileNameLabelMnemonic;
    private String fileNameLabelText;
    private int folderNameLabelMnemonic;
    private String folderNameLabelText;
    private int filesOfTypeLabelMnemonic;
    private String filesOfTypeLabelText;
    private String upFolderToolTipText;
    private String upFolderAccessibleName;
    private String homeFolderToolTipText;
    private String homeFolderAccessibleName;
    private String newFolderToolTipText;
    private String newFolderAccessibleName;
    private String listViewButtonToolTipText;
    private String listViewButtonAccessibleName;
    private String detailsViewButtonToolTipText;
    private String detailsViewButtonAccessibleName;
    private MetalFileChooserUI.AlignedLabel fileNameLabel;
    static final int space = 10;
    
    private void populateFileNameLabel() {
        if (this.getFileChooser().getFileSelectionMode() == 1) {
            this.fileNameLabel.setText(this.folderNameLabelText);
            this.fileNameLabel.setDisplayedMnemonic(this.folderNameLabelMnemonic);
        }
        else {
            this.fileNameLabel.setText(this.fileNameLabelText);
            this.fileNameLabel.setDisplayedMnemonic(this.fileNameLabelMnemonic);
        }
    }
    
    public static ComponentUI createUI(JComponent component) {
        return new MetalFileChooserUI((JFileChooser)component);
    }
    
    public MetalFileChooserUI(JFileChooser b) {
        super(b);
        this.directoryComboBoxAction = new MetalFileChooserUI.DirectoryComboBoxAction();
        this.lookInLabelMnemonic = 0;
        this.lookInLabelText = null;
        this.saveInLabelText = null;
        this.fileNameLabelMnemonic = 0;
        this.fileNameLabelText = null;
        this.folderNameLabelMnemonic = 0;
        this.folderNameLabelText = null;
        this.filesOfTypeLabelMnemonic = 0;
        this.filesOfTypeLabelText = null;
        this.upFolderToolTipText = null;
        this.upFolderAccessibleName = null;
        this.homeFolderToolTipText = null;
        this.homeFolderAccessibleName = null;
        this.newFolderToolTipText = null;
        this.newFolderAccessibleName = null;
        this.listViewButtonToolTipText = null;
        this.listViewButtonAccessibleName = null;
        this.detailsViewButtonToolTipText = null;
        this.detailsViewButtonAccessibleName = null;
    }
    
    @Override
    public void installUI(JComponent c) {
        super.installUI(c);
    }
    
    @Override
    public void uninstallComponents(JFileChooser fileChooser) {
        fileChooser.removeAll();
        this.bottomPanel = null;
        this.buttonPanel = null;
    }
    
    @Override
    public void installComponents(JFileChooser fileChooser) {
        FileSystemView fileSystemView = fileChooser.getFileSystemView();
        fileChooser.setBorder(new EmptyBorder(12, 12, 11, 11));
        fileChooser.setLayout(new BorderLayout(0, 11));
        fileChooser.addPropertyChangeListener(this.filePane = new FilePane(new MetalFileChooserUI.MetalFileChooserUIAccessor()));
        JPanel comp = new JPanel(new BorderLayout(11, 0));
        JPanel panel = new JPanel();
        panel.setLayout(new BoxLayout(panel, 2));
        comp.add(panel, "After");
        fileChooser.add(comp, "North");
        (this.lookInLabel = new JLabel(this.lookInLabelText)).setDisplayedMnemonic(this.lookInLabelMnemonic);
        comp.add(this.lookInLabel, "Before");
        (this.directoryComboBox = new JComboBox() {
            @Override
            public Dimension getPreferredSize() {
                Dimension preferredSize = super.getPreferredSize();
                preferredSize.width = 150;
                return preferredSize;
            }
        }).putClientProperty("AccessibleDescription", this.lookInLabelText);
        this.directoryComboBox.putClientProperty("JComboBox.isTableCellEditor", Boolean.TRUE);
        this.lookInLabel.setLabelFor(this.directoryComboBox);
        this.directoryComboBoxModel = this.createDirectoryComboBoxModel(fileChooser);
        this.directoryComboBox.setModel(this.directoryComboBoxModel);
        this.directoryComboBox.addActionListener(this.directoryComboBoxAction);
        this.directoryComboBox.setRenderer(this.createDirectoryComboBoxRenderer(fileChooser));
        this.directoryComboBox.setAlignmentX(0.0f);
        this.directoryComboBox.setAlignmentY(0.0f);
        this.directoryComboBox.setMaximumRowCount(8);
        comp.add(this.directoryComboBox, "Center");
        JButton comp2 = new JButton(this.getChangeToParentDirectoryAction());
        comp2.setText(null);
        comp2.setIcon(this.upFolderIcon);
        comp2.setToolTipText(this.upFolderToolTipText);
        comp2.putClientProperty("AccessibleName", this.upFolderAccessibleName);
        comp2.setAlignmentX(0.0f);
        comp2.setAlignmentY(0.5f);
        comp2.setMargin(MetalFileChooserUI.shrinkwrap);
        panel.add(comp2);
        panel.add(Box.createRigidArea(MetalFileChooserUI.hstrut5));
        fileSystemView.getHomeDirectory();
        String homeFolderToolTipText = this.homeFolderToolTipText;
        JButton button = new JButton(this.homeFolderIcon);
        button.setToolTipText(homeFolderToolTipText);
        button.putClientProperty("AccessibleName", this.homeFolderAccessibleName);
        button.setAlignmentX(0.0f);
        button.setAlignmentY(0.5f);
        button.setMargin(MetalFileChooserUI.shrinkwrap);
        button.addActionListener(this.getGoHomeAction());
        panel.add(button);
        panel.add(Box.createRigidArea(MetalFileChooserUI.hstrut5));
        if (!UIManager.getBoolean("FileChooser.readOnly")) {
            button = new JButton(this.filePane.getNewFolderAction());
            button.setText(null);
            button.setIcon(this.newFolderIcon);
            button.setToolTipText(this.newFolderToolTipText);
            button.putClientProperty("AccessibleName", this.newFolderAccessibleName);
            button.setAlignmentX(0.0f);
            button.setAlignmentY(0.5f);
            button.setMargin(MetalFileChooserUI.shrinkwrap);
        }
        panel.add(button);
        panel.add(Box.createRigidArea(MetalFileChooserUI.hstrut5));
        ButtonGroup o = new ButtonGroup();
        (this.listViewButton = new JToggleButton(this.listViewIcon)).setToolTipText(this.listViewButtonToolTipText);
        this.listViewButton.putClientProperty("AccessibleName", this.listViewButtonAccessibleName);
        this.listViewButton.setSelected(true);
        this.listViewButton.setAlignmentX(0.0f);
        this.listViewButton.setAlignmentY(0.5f);
        this.listViewButton.setMargin(MetalFileChooserUI.shrinkwrap);
        this.listViewButton.addActionListener(this.filePane.getViewTypeAction(0));
        panel.add(this.listViewButton);
        o.add(this.listViewButton);
        (this.detailsViewButton = new JToggleButton(this.detailsViewIcon)).setToolTipText(this.detailsViewButtonToolTipText);
        this.detailsViewButton.putClientProperty("AccessibleName", this.detailsViewButtonAccessibleName);
        this.detailsViewButton.setAlignmentX(0.0f);
        this.detailsViewButton.setAlignmentY(0.5f);
        this.detailsViewButton.setMargin(MetalFileChooserUI.shrinkwrap);
        this.detailsViewButton.addActionListener(this.filePane.getViewTypeAction(1));
        panel.add(this.detailsViewButton);
        o.add(this.detailsViewButton);
        this.filePane.addPropertyChangeListener(new PropertyChangeListener() {
            @Override
            public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
                if ("viewType".equals(propertyChangeEvent.getPropertyName())) {
                    switch (MetalFileChooserUI.this.filePane.getViewType()) {
                        case 0: {
                            MetalFileChooserUI.this.listViewButton.setSelected(true);
                            break;
                        }
                        case 1: {
                            MetalFileChooserUI.this.detailsViewButton.setSelected(true);
                            break;
                        }
                    }
                }
            }
        });
        fileChooser.add(this.getAccessoryPanel(), "After");
        JComponent accessory = fileChooser.getAccessory();
        if (accessory != null) {
            this.getAccessoryPanel().add(accessory);
        }
        this.filePane.setPreferredSize(MetalFileChooserUI.LIST_PREF_SIZE);
        fileChooser.add(this.filePane, "Center");
        JPanel bottomPanel = this.getBottomPanel();
        bottomPanel.setLayout(new BoxLayout(bottomPanel, 1));
        fileChooser.add(bottomPanel, "South");
        JPanel o2 = new JPanel();
        o2.setLayout(new BoxLayout(o2, 2));
        bottomPanel.add(o2);
        bottomPanel.add(Box.createRigidArea(MetalFileChooserUI.vstrut5));
        this.fileNameLabel = new MetalFileChooserUI.AlignedLabel();
        this.populateFileNameLabel();
        o2.add(this.fileNameLabel);
        o2.add(this.fileNameTextField = new JTextField(35) {
            @Override
            public Dimension getMaximumSize() {
                return new Dimension(32767, super.getPreferredSize().height);
            }
        });
        this.fileNameLabel.setLabelFor(this.fileNameTextField);
        this.fileNameTextField.addFocusListener(new FocusAdapter() {
            @Override
            public void focusGained(FocusEvent focusEvent) {
                if (!MetalFileChooserUI.this.getFileChooser().isMultiSelectionEnabled()) {
                    MetalFileChooserUI.this.filePane.clearSelection();
                }
            }
        });
        if (fileChooser.isMultiSelectionEnabled()) {
            this.setFileName(this.fileNameString(fileChooser.getSelectedFiles()));
        }
        else {
            this.setFileName(this.fileNameString(fileChooser.getSelectedFile()));
        }
        JPanel o3 = new JPanel();
        o3.setLayout(new BoxLayout(o3, 2));
        bottomPanel.add(o3);
        MetalFileChooserUI.AlignedLabel comp3 = new MetalFileChooserUI.AlignedLabel(this.filesOfTypeLabelText);
        comp3.setDisplayedMnemonic(this.filesOfTypeLabelMnemonic);
        o3.add(comp3);
        fileChooser.addPropertyChangeListener(this.filterComboBoxModel = this.createFilterComboBoxModel());
        (this.filterComboBox = new JComboBox(this.filterComboBoxModel)).putClientProperty("AccessibleDescription", this.filesOfTypeLabelText);
        comp3.setLabelFor(this.filterComboBox);
        this.filterComboBox.setRenderer(this.createFilterComboBoxRenderer());
        o3.add(this.filterComboBox);
        this.getButtonPanel().setLayout(new MetalFileChooserUI.ButtonAreaLayout());
        (this.approveButton = new JButton(this.getApproveButtonText(fileChooser))).addActionListener(this.getApproveSelectionAction());
        this.approveButton.setToolTipText(this.getApproveButtonToolTipText(fileChooser));
        this.getButtonPanel().add(this.approveButton);
        (this.cancelButton = new JButton(this.cancelButtonText)).setToolTipText(this.cancelButtonToolTipText);
        this.cancelButton.addActionListener(this.getCancelSelectionAction());
        this.getButtonPanel().add(this.cancelButton);
        if (fileChooser.getControlButtonsAreShown()) {
            this.addControlButtons();
        }
        groupLabels(new MetalFileChooserUI.AlignedLabel[] { this.fileNameLabel, comp3 });
    }
    
    protected JPanel getButtonPanel() {
        if (this.buttonPanel == null) {
            this.buttonPanel = new JPanel();
        }
        return this.buttonPanel;
    }
    
    protected JPanel getBottomPanel() {
        if (this.bottomPanel == null) {
            this.bottomPanel = new JPanel();
        }
        return this.bottomPanel;
    }
    
    @Override
    protected void installStrings(JFileChooser fc) {
        super.installStrings(fc);
        Locale locale = fc.getLocale();
        this.lookInLabelMnemonic = (int)this.getMnemonic("FileChooser.lookInLabelMnemonic", locale);
        this.lookInLabelText = UIManager.getString("FileChooser.lookInLabelText", locale);
        this.saveInLabelText = UIManager.getString("FileChooser.saveInLabelText", locale);
        this.fileNameLabelMnemonic = (int)this.getMnemonic("FileChooser.fileNameLabelMnemonic", locale);
        this.fileNameLabelText = UIManager.getString("FileChooser.fileNameLabelText", locale);
        this.folderNameLabelMnemonic = (int)this.getMnemonic("FileChooser.folderNameLabelMnemonic", locale);
        this.folderNameLabelText = UIManager.getString("FileChooser.folderNameLabelText", locale);
        this.filesOfTypeLabelMnemonic = (int)this.getMnemonic("FileChooser.filesOfTypeLabelMnemonic", locale);
        this.filesOfTypeLabelText = UIManager.getString("FileChooser.filesOfTypeLabelText", locale);
        this.upFolderToolTipText = UIManager.getString("FileChooser.upFolderToolTipText", locale);
        this.upFolderAccessibleName = UIManager.getString("FileChooser.upFolderAccessibleName", locale);
        this.homeFolderToolTipText = UIManager.getString("FileChooser.homeFolderToolTipText", locale);
        this.homeFolderAccessibleName = UIManager.getString("FileChooser.homeFolderAccessibleName", locale);
        this.newFolderToolTipText = UIManager.getString("FileChooser.newFolderToolTipText", locale);
        this.newFolderAccessibleName = UIManager.getString("FileChooser.newFolderAccessibleName", locale);
        this.listViewButtonToolTipText = UIManager.getString("FileChooser.listViewButtonToolTipText", locale);
        this.listViewButtonAccessibleName = UIManager.getString("FileChooser.listViewButtonAccessibleName", locale);
        this.detailsViewButtonToolTipText = UIManager.getString("FileChooser.detailsViewButtonToolTipText", locale);
        this.detailsViewButtonAccessibleName = UIManager.getString("FileChooser.detailsViewButtonAccessibleName", locale);
    }
    
    private Integer getMnemonic(String key, Locale l) {
        return Integer.valueOf(SwingUtilities2.getUIDefaultsInt(key, l));
    }
    
    @Override
    protected void installListeners(JFileChooser fileChooser) {
        super.installListeners(fileChooser);
        SwingUtilities.replaceUIActionMap(fileChooser, this.getActionMap());
    }
    
    protected ActionMap getActionMap() {
        return this.createActionMap();
    }
    
    protected ActionMap createActionMap() {
        ActionMapUIResource map = new ActionMapUIResource();
        FilePane.addActionsToMap(map, this.filePane.getActions());
        return map;
    }
    
    protected JPanel createList(JFileChooser fileChooser) {
        return this.filePane.createList();
    }
    
    protected JPanel createDetailsView(JFileChooser fileChooser) {
        return this.filePane.createDetailsView();
    }
    
    @Override
    public ListSelectionListener createListSelectionListener(JFileChooser fc) {
        return super.createListSelectionListener(fc);
    }
    
    @Override
    public void uninstallUI(JComponent c) {
        c.removePropertyChangeListener(this.filterComboBoxModel);
        c.removePropertyChangeListener(this.filePane);
        this.cancelButton.removeActionListener(this.getCancelSelectionAction());
        this.approveButton.removeActionListener(this.getApproveSelectionAction());
        this.fileNameTextField.removeActionListener(this.getApproveSelectionAction());
        if (this.filePane != null) {
            this.filePane.uninstallUI();
            this.filePane = null;
        }
        super.uninstallUI(c);
    }
    
    @Override
    public Dimension getPreferredSize(JComponent component) {
        int width = MetalFileChooserUI.PREF_SIZE.width;
        Dimension preferredLayoutSize = component.getLayout().preferredLayoutSize(component);
        if (preferredLayoutSize != null) {
            return new Dimension((preferredLayoutSize.width < width) ? width : preferredLayoutSize.width, (preferredLayoutSize.height < MetalFileChooserUI.PREF_SIZE.height) ? MetalFileChooserUI.PREF_SIZE.height : preferredLayoutSize.height);
        }
        return new Dimension(width, MetalFileChooserUI.PREF_SIZE.height);
    }
    
    @Override
    public Dimension getMinimumSize(JComponent component) {
        return new Dimension(MetalFileChooserUI.MIN_WIDTH, MetalFileChooserUI.MIN_HEIGHT);
    }
    
    @Override
    public Dimension getMaximumSize(JComponent component) {
        return new Dimension(Integer.MAX_VALUE, Integer.MAX_VALUE);
    }
    
    private String fileNameString(File dir) {
        if (dir == null) {
            return null;
        }
        JFileChooser fileChooser = this.getFileChooser();
        if ((fileChooser.isDirectorySelectionEnabled() && !fileChooser.isFileSelectionEnabled()) || (fileChooser.isDirectorySelectionEnabled() && fileChooser.isFileSelectionEnabled() && fileChooser.getFileSystemView().isFileSystemRoot(dir))) {
            return dir.getPath();
        }
        return dir.getName();
    }
    
    private String fileNameString(File[] array) {
        StringBuffer sb = new StringBuffer();
        for (int n = 0; array != null && n < array.length; ++n) {
            if (n > 0) {
                sb.append(" ");
            }
            if (array.length > 1) {
                sb.append("\"");
            }
            sb.append(this.fileNameString(array[n]));
            if (array.length > 1) {
                sb.append("\"");
            }
        }
        return sb.toString();
    }
    
    private void doSelectedFileChanged(PropertyChangeEvent propertyChangeEvent) {
        File file = (File)propertyChangeEvent.getNewValue();
        JFileChooser fileChooser = this.getFileChooser();
        if (file != null && ((fileChooser.isFileSelectionEnabled() && !file.isDirectory()) || (file.isDirectory() && fileChooser.isDirectorySelectionEnabled()))) {
            this.setFileName(this.fileNameString(file));
        }
    }
    
    private void doSelectedFilesChanged(PropertyChangeEvent propertyChangeEvent) {
        File[] array = (File[])(File[])propertyChangeEvent.getNewValue();
        JFileChooser fileChooser = this.getFileChooser();
        if (array != null && array.length > 0 && (array.length > 1 || fileChooser.isDirectorySelectionEnabled() || !array[0].isDirectory())) {
            this.setFileName(this.fileNameString(array));
        }
    }
    
    private void doDirectoryChanged(PropertyChangeEvent propertyChangeEvent) {
        JFileChooser fileChooser = this.getFileChooser();
        FileSystemView fileSystemView = fileChooser.getFileSystemView();
        this.clearIconCache();
        File currentDirectory = fileChooser.getCurrentDirectory();
        if (currentDirectory != null) {
            MetalFileChooserUI.DirectoryComboBoxModel.access$800(this.directoryComboBoxModel, currentDirectory);
            if (fileChooser.isDirectorySelectionEnabled() && !fileChooser.isFileSelectionEnabled()) {
                if (fileSystemView.isFileSystem(currentDirectory)) {
                    this.setFileName(currentDirectory.getPath());
                }
                else {
                    this.setFileName(null);
                }
            }
        }
    }
    
    private void doFilterChanged(PropertyChangeEvent propertyChangeEvent) {
        this.clearIconCache();
    }
    
    private void doFileSelectionModeChanged(PropertyChangeEvent propertyChangeEvent) {
        if (this.fileNameLabel != null) {
            this.populateFileNameLabel();
        }
        this.clearIconCache();
        JFileChooser fileChooser = this.getFileChooser();
        File currentDirectory = fileChooser.getCurrentDirectory();
        if (currentDirectory != null && fileChooser.isDirectorySelectionEnabled() && !fileChooser.isFileSelectionEnabled() && fileChooser.getFileSystemView().isFileSystem(currentDirectory)) {
            this.setFileName(currentDirectory.getPath());
        }
        else {
            this.setFileName(null);
        }
    }
    
    private void doAccessoryChanged(PropertyChangeEvent propertyChangeEvent) {
        if (this.getAccessoryPanel() != null) {
            if (propertyChangeEvent.getOldValue() != null) {
                this.getAccessoryPanel().remove((JComponent)propertyChangeEvent.getOldValue());
            }
            JComponent comp = (JComponent)propertyChangeEvent.getNewValue();
            if (comp != null) {
                this.getAccessoryPanel().add(comp, "Center");
            }
        }
    }
    
    private void doApproveButtonTextChanged(PropertyChangeEvent propertyChangeEvent) {
        JFileChooser fileChooser = this.getFileChooser();
        this.approveButton.setText(this.getApproveButtonText(fileChooser));
        this.approveButton.setToolTipText(this.getApproveButtonToolTipText(fileChooser));
    }
    
    private void doDialogTypeChanged(PropertyChangeEvent propertyChangeEvent) {
        JFileChooser fileChooser = this.getFileChooser();
        this.approveButton.setText(this.getApproveButtonText(fileChooser));
        this.approveButton.setToolTipText(this.getApproveButtonToolTipText(fileChooser));
        if (fileChooser.getDialogType() == 1) {
            this.lookInLabel.setText(this.saveInLabelText);
        }
        else {
            this.lookInLabel.setText(this.lookInLabelText);
        }
    }
    
    private void doApproveButtonMnemonicChanged(PropertyChangeEvent propertyChangeEvent) {
    }
    
    private void doControlButtonsChanged(PropertyChangeEvent propertyChangeEvent) {
        if (this.getFileChooser().getControlButtonsAreShown()) {
            this.addControlButtons();
        }
        else {
            this.removeControlButtons();
        }
    }
    
    @Override
    public PropertyChangeListener createPropertyChangeListener(JFileChooser fileChooser) {
        return new PropertyChangeListener() {
            @Override
            public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
                String propertyName = propertyChangeEvent.getPropertyName();
                if (propertyName.equals("SelectedFileChangedProperty")) {
                    MetalFileChooserUI.access$900(MetalFileChooserUI.this, propertyChangeEvent);
                }
                else if (propertyName.equals("SelectedFilesChangedProperty")) {
                    MetalFileChooserUI.access$1000(MetalFileChooserUI.this, propertyChangeEvent);
                }
                else if (propertyName.equals("directoryChanged")) {
                    MetalFileChooserUI.access$1100(MetalFileChooserUI.this, propertyChangeEvent);
                }
                else if (propertyName.equals("fileFilterChanged")) {
                    MetalFileChooserUI.access$1200(MetalFileChooserUI.this, propertyChangeEvent);
                }
                else if (propertyName.equals("fileSelectionChanged")) {
                    MetalFileChooserUI.access$1300(MetalFileChooserUI.this, propertyChangeEvent);
                }
                else if (propertyName.equals("AccessoryChangedProperty")) {
                    MetalFileChooserUI.access$1400(MetalFileChooserUI.this, propertyChangeEvent);
                }
                else if (propertyName.equals("ApproveButtonTextChangedProperty") || propertyName.equals("ApproveButtonToolTipTextChangedProperty")) {
                    MetalFileChooserUI.access$1500(MetalFileChooserUI.this, propertyChangeEvent);
                }
                else if (propertyName.equals("DialogTypeChangedProperty")) {
                    MetalFileChooserUI.access$1600(MetalFileChooserUI.this, propertyChangeEvent);
                }
                else if (propertyName.equals("ApproveButtonMnemonicChangedProperty")) {
                    MetalFileChooserUI.access$1700(MetalFileChooserUI.this, propertyChangeEvent);
                }
                else if (propertyName.equals("ControlButtonsAreShownChangedProperty")) {
                    MetalFileChooserUI.access$1800(MetalFileChooserUI.this, propertyChangeEvent);
                }
                else if (propertyName.equals("componentOrientation")) {
                    ComponentOrientation o = (ComponentOrientation)propertyChangeEvent.getNewValue();
                    JFileChooser fileChooser = (JFileChooser)propertyChangeEvent.getSource();
                    if (o != propertyChangeEvent.getOldValue()) {
                        fileChooser.applyComponentOrientation(o);
                    }
                }
                else if (propertyName == "FileChooser.useShellFolder") {
                    MetalFileChooserUI.access$1100(MetalFileChooserUI.this, propertyChangeEvent);
                }
                else if (propertyName.equals("ancestor") && propertyChangeEvent.getOldValue() == null && propertyChangeEvent.getNewValue() != null) {
                    MetalFileChooserUI.this.fileNameTextField.selectAll();
                    MetalFileChooserUI.this.fileNameTextField.requestFocus();
                }
            }
        };
    }
    
    protected void removeControlButtons() {
        this.getBottomPanel().remove(this.getButtonPanel());
    }
    
    protected void addControlButtons() {
        this.getBottomPanel().add(this.getButtonPanel());
    }
    
    @Override
    public void ensureFileIsVisible(JFileChooser fc, File f) {
        this.filePane.ensureFileIsVisible(fc, f);
    }
    
    @Override
    public void rescanCurrentDirectory(JFileChooser fileChooser) {
        this.filePane.rescanCurrentDirectory();
    }
    
    @Override
    public String getFileName() {
        if (this.fileNameTextField != null) {
            return this.fileNameTextField.getText();
        }
        return null;
    }
    
    @Override
    public void setFileName(String text) {
        if (this.fileNameTextField != null) {
            this.fileNameTextField.setText(text);
        }
    }
    
    @Override
    protected void setDirectorySelected(boolean directorySelected) {
        super.setDirectorySelected(directorySelected);
        JFileChooser fileChooser = this.getFileChooser();
        if (directorySelected) {
            if (this.approveButton != null) {
                this.approveButton.setText(this.directoryOpenButtonText);
                this.approveButton.setToolTipText(this.directoryOpenButtonToolTipText);
            }
        }
        else if (this.approveButton != null) {
            this.approveButton.setText(this.getApproveButtonText(fileChooser));
            this.approveButton.setToolTipText(this.getApproveButtonToolTipText(fileChooser));
        }
    }
    
    @Override
    public String getDirectoryName() {
        return null;
    }
    
    @Override
    public void setDirectoryName(String s) {
    }
    
    protected MetalFileChooserUI.DirectoryComboBoxRenderer createDirectoryComboBoxRenderer(JFileChooser fileChooser) {
        return new MetalFileChooserUI.DirectoryComboBoxRenderer();
    }
    
    protected MetalFileChooserUI.DirectoryComboBoxModel createDirectoryComboBoxModel(JFileChooser fileChooser) {
        return new MetalFileChooserUI.DirectoryComboBoxModel();
    }
    
    protected MetalFileChooserUI.FilterComboBoxRenderer createFilterComboBoxRenderer() {
        return new MetalFileChooserUI.FilterComboBoxRenderer();
    }
    
    protected MetalFileChooserUI.FilterComboBoxModel createFilterComboBoxModel() {
        return new MetalFileChooserUI.FilterComboBoxModel();
    }
    
    public void valueChanged(ListSelectionEvent listSelectionEvent) {
        File selectedFile = this.getFileChooser().getSelectedFile();
        if (!listSelectionEvent.getValueIsAdjusting() && selectedFile != null && !this.getFileChooser().isTraversable(selectedFile)) {
            this.setFileName(this.fileNameString(selectedFile));
        }
    }
    
    @Override
    protected JButton getApproveButton(JFileChooser fileChooser) {
        return this.approveButton;
    }
    
    private static void groupLabels(MetalFileChooserUI.AlignedLabel[] array) {
        for (int i = 0; i < array.length; ++i) {
            array[i].group = array;
        }
    }
    
    static boolean access$000(MetalFileChooserUI metalFileChooserUI) {
        return metalFileChooserUI.isDirectorySelected();
    }
    
    static File access$100(MetalFileChooserUI metalFileChooserUI) {
        return metalFileChooserUI.getDirectory();
    }
    
    static MouseListener access$200(MetalFileChooserUI metalFileChooserUI, JFileChooser fc, JList list) {
        return metalFileChooserUI.createDoubleClickListener(fc, list);
    }
    
    static void access$900(MetalFileChooserUI metalFileChooserUI, PropertyChangeEvent propertyChangeEvent) {
        metalFileChooserUI.doSelectedFileChanged(propertyChangeEvent);
    }
    
    static void access$1000(MetalFileChooserUI metalFileChooserUI, PropertyChangeEvent propertyChangeEvent) {
        metalFileChooserUI.doSelectedFilesChanged(propertyChangeEvent);
    }
    
    static void access$1100(MetalFileChooserUI metalFileChooserUI, PropertyChangeEvent propertyChangeEvent) {
        metalFileChooserUI.doDirectoryChanged(propertyChangeEvent);
    }
    
    static void access$1200(MetalFileChooserUI metalFileChooserUI, PropertyChangeEvent propertyChangeEvent) {
        metalFileChooserUI.doFilterChanged(propertyChangeEvent);
    }
    
    static void access$1300(MetalFileChooserUI metalFileChooserUI, PropertyChangeEvent propertyChangeEvent) {
        metalFileChooserUI.doFileSelectionModeChanged(propertyChangeEvent);
    }
    
    static void access$1400(MetalFileChooserUI metalFileChooserUI, PropertyChangeEvent propertyChangeEvent) {
        metalFileChooserUI.doAccessoryChanged(propertyChangeEvent);
    }
    
    static void access$1500(MetalFileChooserUI metalFileChooserUI, PropertyChangeEvent propertyChangeEvent) {
        metalFileChooserUI.doApproveButtonTextChanged(propertyChangeEvent);
    }
    
    static void access$1600(MetalFileChooserUI metalFileChooserUI, PropertyChangeEvent propertyChangeEvent) {
        metalFileChooserUI.doDialogTypeChanged(propertyChangeEvent);
    }
    
    static void access$1700(MetalFileChooserUI metalFileChooserUI, PropertyChangeEvent propertyChangeEvent) {
        metalFileChooserUI.doApproveButtonMnemonicChanged(propertyChangeEvent);
    }
    
    static void access$1800(MetalFileChooserUI metalFileChooserUI, PropertyChangeEvent propertyChangeEvent) {
        metalFileChooserUI.doControlButtonsChanged(propertyChangeEvent);
    }
    
    static {
        hstrut5 = new Dimension(5, 1);
        hstrut11 = new Dimension(11, 1);
        vstrut5 = new Dimension(1, 5);
        shrinkwrap = new Insets(0, 0, 0, 0);
        MetalFileChooserUI.PREF_WIDTH = 500;
        MetalFileChooserUI.PREF_HEIGHT = 326;
        MetalFileChooserUI.PREF_SIZE = new Dimension(MetalFileChooserUI.PREF_WIDTH, MetalFileChooserUI.PREF_HEIGHT);
        MetalFileChooserUI.MIN_WIDTH = 500;
        MetalFileChooserUI.MIN_HEIGHT = 326;
        MetalFileChooserUI.LIST_PREF_WIDTH = 405;
        MetalFileChooserUI.LIST_PREF_HEIGHT = 135;
        MetalFileChooserUI.LIST_PREF_SIZE = new Dimension(MetalFileChooserUI.LIST_PREF_WIDTH, MetalFileChooserUI.LIST_PREF_HEIGHT);
    }
    
    private class MetalFileChooserUIAccessor implements FilePane.FileChooserUIAccessor
    {
        @Override
        public JFileChooser getFileChooser() {
            return MetalFileChooserUI.this.getFileChooser();
        }
        
        @Override
        public BasicDirectoryModel getModel() {
            return MetalFileChooserUI.this.getModel();
        }
        
        @Override
        public JPanel createList() {
            return MetalFileChooserUI.this.createList(this.getFileChooser());
        }
        
        @Override
        public JPanel createDetailsView() {
            return MetalFileChooserUI.this.createDetailsView(this.getFileChooser());
        }
        
        @Override
        public boolean isDirectorySelected() {
            return MetalFileChooserUI.access$000(MetalFileChooserUI.this);
        }
        
        @Override
        public File getDirectory() {
            return MetalFileChooserUI.access$100(MetalFileChooserUI.this);
        }
        
        @Override
        public Action getChangeToParentDirectoryAction() {
            return MetalFileChooserUI.this.getChangeToParentDirectoryAction();
        }
        
        @Override
        public Action getApproveSelectionAction() {
            return MetalFileChooserUI.this.getApproveSelectionAction();
        }
        
        @Override
        public Action getNewFolderAction() {
            return MetalFileChooserUI.this.getNewFolderAction();
        }
        
        @Override
        public MouseListener createDoubleClickListener(JList list) {
            return MetalFileChooserUI.access$200(MetalFileChooserUI.this, this.getFileChooser(), list);
        }
        
        @Override
        public ListSelectionListener createListSelectionListener() {
            return MetalFileChooserUI.this.createListSelectionListener(this.getFileChooser());
        }
    }
    
    protected class SingleClickListener extends MouseAdapter
    {
        public SingleClickListener(JList list) {
        }
    }
    
    protected class FileRenderer extends DefaultListCellRenderer
    {
    }
    
    class DirectoryComboBoxRenderer extends DefaultListCellRenderer
    {
        MetalFileChooserUI.IndentIcon ii;
        
        DirectoryComboBoxRenderer() {
            this.ii = new MetalFileChooserUI.IndentIcon();
        }
        
        @Override
        public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
            super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
            if (value == null) {
                this.setText("");
                return this;
            }
            File file = (File)value;
            this.setText(MetalFileChooserUI.this.getFileChooser().getName(file));
            this.ii.icon = MetalFileChooserUI.this.getFileChooser().getIcon(file);
            this.ii.depth = MetalFileChooserUI.this.directoryComboBoxModel.getDepth(index);
            this.setIcon(this.ii);
            return this;
        }
    }
    
    class IndentIcon implements Icon
    {
        Icon icon;
        int depth;
        
        IndentIcon() {
            this.icon = null;
            this.depth = 0;
        }
        
        @Override
        public void paintIcon(Component component, Graphics graphics, int n, int n2) {
            if (component.getComponentOrientation().isLeftToRight()) {
                this.icon.paintIcon(component, graphics, n + this.depth * 10, n2);
            }
            else {
                this.icon.paintIcon(component, graphics, n, n2);
            }
        }
        
        @Override
        public int getIconWidth() {
            return this.icon.getIconWidth() + this.depth * 10;
        }
        
        @Override
        public int getIconHeight() {
            return this.icon.getIconHeight();
        }
    }
    
    protected class DirectoryComboBoxModel extends AbstractListModel<Object> implements ComboBoxModel<Object>
    {
        Vector<File> directories;
        int[] depths;
        File selectedDirectory;
        JFileChooser chooser;
        FileSystemView fsv;
        
        public DirectoryComboBoxModel() {
            this.directories = new Vector();
            this.depths = null;
            this.selectedDirectory = null;
            this.chooser = MetalFileChooserUI.this.getFileChooser();
            this.fsv = this.chooser.getFileSystemView();
            File currentDirectory = MetalFileChooserUI.this.getFileChooser().getCurrentDirectory();
            if (currentDirectory != null) {
                this.addItem(currentDirectory);
            }
        }
        
        private void addItem(File f) {
            if (f == null) {
                return;
            }
            int usesShellFolder = FilePane.usesShellFolder(this.chooser);
            this.directories.clear();
            this.directories.addAll(Arrays.asList(usesShellFolder ? ((File[])(File[])ShellFolder.get("fileChooserComboBoxFolders")) : this.fsv.getRoots()));
            File selectedItem;
            try {
                selectedItem = ShellFolder.getNormalizedFile(f);
            }
            catch (IOException ex) {
                selectedItem = f;
            }
            try {
                selectedItem = (selectedItem = (usesShellFolder ? ShellFolder.getShellFolder(selectedItem) : selectedItem));
                Vector o = new Vector(10);
                do {
                    o.addElement(selectedItem);
                } while ((selectedItem = selectedItem.getParentFile()) != null);
                int size = o.size();
                int i = 0;
                while (i < size) {
                    File file = (File)o.get(i);
                    if (this.directories.contains(file)) {
                        int index = this.directories.indexOf(file);
                        for (int j = i - 1; j >= 0; --j) {
                            this.directories.insertElementAt(o.get(j), index + i - j);
                        }
                        break;
                    }
                    ++i;
                }
                this.calculateDepths();
                this.setSelectedItem(selectedItem);
            }
            catch (FileNotFoundException ex2) {
                this.calculateDepths();
            }
            return;
        }
        
        private void calculateDepths() {
            this.depths = new int[this.directories.size()];
            for (int i = 0; i < this.depths.length; ++i) {
                File parentFile = ((File)this.directories.get(i)).getParentFile();
                this.depths[i] = 0;
                if (parentFile != null) {
                    int j = i - 1;
                    while (j >= 0) {
                        if (parentFile.equals(this.directories.get(j))) {
                            this.depths[i] = this.depths[j] + 1;
                            break;
                        }
                        --j;
                    }
                }
            }
        }
        
        public int getDepth(int n) {
            return (this.depths != null && n >= 0 && n < this.depths.length) ? this.depths[n] : 0;
        }
        
        @Override
        public void setSelectedItem(Object o) {
            this.selectedDirectory = (File)o;
            this.fireContentsChanged(this, -1, -1);
        }
        
        @Override
        public Object getSelectedItem() {
            return this.selectedDirectory;
        }
        
        @Override
        public int getSize() {
            return this.directories.size();
        }
        
        @Override
        public Object getElementAt(int index) {
            return this.directories.elementAt(index);
        }
        
        static void access$800(MetalFileChooserUI.DirectoryComboBoxModel directoryComboBoxModel, File file) {
            directoryComboBoxModel.addItem(file);
        }
    }
    
    public class FilterComboBoxRenderer extends DefaultListCellRenderer
    {
        @Override
        public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
            super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
            if (value != null && value instanceof FileFilter) {
                this.setText(((FileFilter)value).getDescription());
                return this;
            }
            return this;
        }
    }
    
    protected class FilterComboBoxModel extends AbstractListModel<Object> implements ComboBoxModel<Object>, PropertyChangeListener
    {
        protected FileFilter[] filters;
        
        protected FilterComboBoxModel() {
            this.filters = MetalFileChooserUI.this.getFileChooser().getChoosableFileFilters();
        }
        
        @Override
        public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
            String propertyName = propertyChangeEvent.getPropertyName();
            if (propertyName == "ChoosableFileFilterChangedProperty") {
                this.filters = (FileFilter[])(FileFilter[])propertyChangeEvent.getNewValue();
                this.fireContentsChanged(this, -1, -1);
            }
            else if (propertyName == "fileFilterChanged") {
                this.fireContentsChanged(this, -1, -1);
            }
        }
        
        @Override
        public void setSelectedItem(Object o) {
            if (o != null) {
                MetalFileChooserUI.this.getFileChooser().setFileFilter((FileFilter)o);
                this.fireContentsChanged(this, -1, -1);
            }
        }
        
        @Override
        public Object getSelectedItem() {
            FileFilter fileFilter = MetalFileChooserUI.this.getFileChooser().getFileFilter();
            boolean n = false;
            if (fileFilter != null) {
                FileFilter[] filters = this.filters;
                for (int length = filters.length, i = 0; i < length; ++i) {
                    if (filters[i] == fileFilter) {
                        n = true;
                    }
                }
                if (n == 0) {
                    MetalFileChooserUI.this.getFileChooser().addChoosableFileFilter(fileFilter);
                }
            }
            return MetalFileChooserUI.this.getFileChooser().getFileFilter();
        }
        
        @Override
        public int getSize() {
            if (this.filters != null) {
                return this.filters.length;
            }
            return 0;
        }
        
        @Override
        public Object getElementAt(int n) {
            if (n > this.getSize() - 1) {
                return MetalFileChooserUI.this.getFileChooser().getFileFilter();
            }
            if (this.filters != null) {
                return this.filters[n];
            }
            return null;
        }
    }
    
    protected class DirectoryComboBoxAction extends AbstractAction
    {
        protected DirectoryComboBoxAction() {
            super("DirectoryComboBoxAction");
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            MetalFileChooserUI.this.directoryComboBox.hidePopup();
            File file = (File)MetalFileChooserUI.this.directoryComboBox.getSelectedItem();
            if (!MetalFileChooserUI.this.getFileChooser().getCurrentDirectory().equals(file)) {
                MetalFileChooserUI.this.getFileChooser().setCurrentDirectory(file);
            }
        }
    }
    
    private static class ButtonAreaLayout implements LayoutManager
    {
        private int hGap;
        private int topMargin;
        
        private ButtonAreaLayout() {
            this.hGap = 5;
            this.topMargin = 17;
        }
        
        @Override
        public void addLayoutComponent(String s, Component component) {
        }
        
        @Override
        public void layoutContainer(Container container) {
            Component[] components = container.getComponents();
            if (components != null && components.length > 0) {
                int length = components.length;
                Dimension[] array = new Dimension[length];
                Insets insets = container.getInsets();
                int y = insets.top + this.topMargin;
                int max = 0;
                for (int i = 0; i < length; ++i) {
                    array[i] = components[i].getPreferredSize();
                    max = Math.max(max, array[i].width);
                }
                int left;
                int n;
                if (container.getComponentOrientation().isLeftToRight()) {
                    left = container.getSize().width - insets.left - max;
                    n = this.hGap + max;
                }
                else {
                    left = insets.left;
                    n = -(this.hGap + max);
                }
                for (int j = length - 1; j >= 0; left -= n, --j) {
                    components[j].setBounds(left, y, max, array[j].height);
                }
            }
        }
        
        @Override
        public Dimension minimumLayoutSize(Container container) {
            if (container != null) {
                Component[] components = container.getComponents();
                if (components != null && components.length > 0) {
                    int length = components.length;
                    int max = 0;
                    Insets insets = container.getInsets();
                    int n = this.topMargin + insets.top + insets.bottom;
                    int n2 = insets.left + insets.right;
                    int max2 = 0;
                    for (int i = 0; i < length; ++i) {
                        Dimension preferredSize = components[i].getPreferredSize();
                        max = Math.max(max, preferredSize.height);
                        max2 = Math.max(max2, preferredSize.width);
                    }
                    return new Dimension(n2 + length * max2 + (length - 1) * this.hGap, n + max);
                }
            }
            return new Dimension(0, 0);
        }
        
        @Override
        public Dimension preferredLayoutSize(Container container) {
            return this.minimumLayoutSize(container);
        }
        
        @Override
        public void removeLayoutComponent(Component component) {
        }
    }
    
    private class AlignedLabel extends JLabel
    {
        private MetalFileChooserUI.AlignedLabel[] group;
        private int maxWidth;
        
        AlignedLabel() {
            this.maxWidth = 0;
            this.setAlignmentX(0.0f);
        }
        
        AlignedLabel(String text) {
            super(text);
            this.maxWidth = 0;
            this.setAlignmentX(0.0f);
        }
        
        @Override
        public Dimension getPreferredSize() {
            return new Dimension(this.getMaxWidth() + 11, super.getPreferredSize().height);
        }
        
        private int getMaxWidth() {
            if (this.maxWidth == 0 && this.group != null) {
                int max = 0;
                for (int i = 0; i < this.group.length; ++i) {
                    max = Math.max(this.group[i].getSuperPreferredWidth(), max);
                }
                for (int j = 0; j < this.group.length; ++j) {
                    this.group[j].maxWidth = max;
                }
            }
            return this.maxWidth;
        }
        
        private int getSuperPreferredWidth() {
            return super.getPreferredSize().width;
        }
    }
}
